@@ -1,14 +1,14 @@
-# Copyrights 2006-2014 by [Mark Overmeer].
+# Copyrights 2006-2016 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
-# Pod stripped from pm file by OODoc 2.01.
+# Pod stripped from pm file by OODoc 2.02.
 use warnings;
 use strict;
 no warnings 'recursion';  # trees can be quite deep
 
 package XML::Compile::Translate;
 use vars '$VERSION';
-$VERSION = '1.47';
+$VERSION = '1.52';
 
 
 # Errors are either in _class 'usage': called with request
@@ -76,7 +76,7 @@ sub actsAs($) { panic "not implemented" }
 
 sub compile($@)
 {   my ($self, $item, %args) = @_;
-    @$self{keys %args} = values %args;  # dirty
+    @$self{keys %args} = values %args;  # dirty.  Always all the same fields
 
     my $path   = $self->prefixed($item, 1) || $item;
     ref $item
@@ -103,7 +103,7 @@ sub compile($@)
     }
 
     delete $self->{_created};
-    my $produce = $self->topLevel($path, $item);
+    my $produce = $self->topLevel($path, $item, 1);
     delete $self->{_created};
 
     my $in = $self->{include_namespaces}
@@ -197,8 +197,8 @@ sub nsContext($)
 
 sub namespaces() { $_[0]->{nss} }
 
-sub topLevel($$)
-{   my ($self, $path, $fullname) = @_;
+sub topLevel($$;$)
+{   my ($self, $path, $fullname, $is_root) = @_;
 
     # built-in types have to be handled differently.
     my $internal = XML::Compile::Schema::Specs->builtInType(undef, $fullname
@@ -238,7 +238,7 @@ sub topLevel($$)
     my $name = $node->localName;
     my $data;
     if($name eq 'element')
-    {   my ($label, $make) = $self->element($tree);
+    {   my ($label, $make) = $self->element($tree, $is_root);
         $data    = $self->makeElementWrapper($path, $make) if $make;
     }
     elsif($name eq 'attribute')
@@ -525,8 +525,8 @@ sub applySimpleFacets($$$$)
     : $self->makeFacets($where, $st, \%facets_info, @early, @late);
 }
 
-sub element($)
-{   my ($self, $tree) = @_;
+sub element($;$)
+{   my ($self, $tree, $is_root) = @_;
 
     # attributes: abstract, default, fixed, form, id, maxOccurs, minOccurs
     #    , name, nillable, ref, substitutionGroup, targetNamespace, type
@@ -587,19 +587,39 @@ sub element($)
     local $self->{_context} = $context if $is_global;
     my $nodetype = $qual ? $fullname : $name;
 
+    # SubstitionGroups
+    # We know the type of the message root, so do not need to look for
+    # alternative sgs (and it wouldn't work anyway)
+
+    my @sgs;
+    @sgs = $self->namespaces->findSgMembers($node->localName, $fullname)
+        unless $is_root;
+
     # Handle re-usable fragments, fight against combinatorial explosions
 
     my $nodeid   = $node->unique_key; #$node->nodePath.'#'.$fullname;
-    my $already  = $self->{_created}{$nodeid};
-#warn "$nodeid; ", $node+0,"\n" if $already;
-#undef $already;
-    return ($nodetype, $already) if $already;
+    if(my $already  = $self->{_created}{$nodeid})
+    {    # We cannot cache compile subst-group handlers, because sgs using
+         # elements which were already compiled into sgs does not work.
+         $already = $self->substitutionGroup($tree, $fullname, $nodetype
+           , $already, \@sgs) if @sgs;
+         return ($nodetype, $already);
+    }
 
     # Detect recursion
+    # Very complicated: recursively nested structures.  It is less of a
+    # problem when you handle in run-time what you see... but we here
+    # have to be prepared for everything.
 
     if(exists $self->{_nest}{$nodeid})
-    {   my $outer = \$self->{_nest}{$nodeid};
-        return ($nodetype, sub { $$outer->(@_) });
+    {   my $outer  = \$self->{_nest}{$nodeid};
+        my $nested = sub { $$outer->(@_) };
+
+        # The code must be blessed in the right class, to be compiled
+        # correctly inside its parent.
+        bless $nested, 'BLOCK' if @sgs;
+
+        return ($nodetype, $nested);
     }
     $self->{_nest}{$nodeid} = undef;
 
@@ -623,16 +643,21 @@ sub element($)
                  || $self->typeByName($where, $tree, $comptype);
     }
     elsif($nr_childs==0)
-    {   if(my $subst = $node->getAttribute('substitutionGroup'))
-        {   # default type for substGroups is type of base-class
-            my $subst_elem = $self->rel2abs($where, $node, $subst);
+    {   # default type for substGroups is type of base-class
+        my $base_node = $node;
+        local $self->{_context};
+        while(my $subst = $base_node->getAttribute('substitutionGroup'))
+        {   my $subst_elem = $self->rel2abs($where, $base_node, $subst);
             my $base_elem  = $self->namespaces->find(element => $subst_elem);
-            my $node       = $base_elem->{node};
-            if(my $isa = $node->getAttribute('type'))
-            {   $comptype  = $self->rel2abs($where, $node, $isa);
-                $comps     = $self->blocked($where, complexType => $comptype)
-                          || $self->typeByName($where, $tree, $comptype);
-            }
+            $self->{_context} = $self->nsContext($base_elem);
+            $base_node     = $base_elem->{node};
+            my $isa        = $base_node->getAttribute('type')
+                or next;
+
+            $comptype  = $self->rel2abs($where, $base_node, $isa);
+            $comps     = $self->blocked($where, complexType => $comptype)
+                      || $self->typeByName($where, $tree, $comptype);
+            last;
         }
         unless($comptype)
         {   # no type found, so anyType
@@ -695,43 +720,38 @@ sub element($)
       : $fixed    ? 'makeElementFixed'
       :             'makeElement';
 
-    my $do1 = $self->$generate($where, $ns, $nodetype, $r, $value, $tag);
+    my $do = $self->$generate($where, $ns, $nodetype, $r, $value, $tag);
 
     # hrefs are used by SOAP-RPC
-    my $do2 = $self->{permit_href} && $self->actsAs('READER')
-      ? $self->makeElementHref($where, $ns, $nodetype, $do1) : $do1;
+    $do = $self->makeElementHref($where, $ns, $nodetype, $do)
+        if $self->{permit_href} && $self->actsAs('READER');
 
     # Implement hooks
     my ($before, $replace, $after)
       = $self->findHooks($where, $comptype, $node);
 
-    my $do3
-      = ($before || $replace || $after)
-      ? $self->makeHook($where, $do2, $tag, $before, $replace, $after)
-      : $do2;
+    $do = $self->makeHook($where,$do,$tag,$before,$replace,$after,$comptype)
+        if $before || $replace || $after;
 
-    my $do4 = $do3;
-    if($comptype && $self->{xsi_type}{$comptype})
-    {   # Ugly xsi:type switch needed
-        $do4 = $self->xsiType($tree, $node, $name, $comptype, $do3);
-    }
+    $do = $self->xsiType($tree, $node, $name, $comptype, $do)
+        if $comptype && $self->{xsi_type}{$comptype};
 
-    my $do5 = $do4;
-    if($is_global)
-    {   my @sgs = $self->namespaces->findSgMembers($node->localName, $fullname);
-        $do5 = $self->substitutionGroup($tree, $fullname, $nodetype, $do4,\@sgs)
-            if @sgs;
-    }
+    $do = $self->addTypeAttribute($comptype, $do)
+        if $self->{xsi_type_everywhere} && $comptype !~ /^unnamed /;
+
+    $self->{_created}{$nodeid} = $do;
+
+    $do = $self->substitutionGroup($tree, $fullname, $nodetype, $do, \@sgs)
+        if @sgs;
 
     # handle recursion
     # this must look very silly to you... however, this is resolving
     # recursive schemas: this way nested use of the same element
     # definition will catch the code reference of the outer definition.
-    $self->{_nest}{$nodeid}    = $do5;
+    $self->{_nest}{$nodeid} = $do;
     delete $self->{_nest}{$nodeid};  # clean the outer definition
 
-    $self->{_created}{$nodeid} = $do5;
-    ($nodetype, $do5);
+    ($nodetype, $do);
 }
 
 sub particle($)
@@ -932,7 +952,7 @@ sub substitutionGroup($$$$$)
     push @elems, $label => [$self->keyRewrite($label), $base] if $base;
 
     foreach my $subst (@$sgs)
-    {   my ($l, $d) = $self->element($tree->descend($subst->{node}));
+    {   my ($l, $d) = $self->element($tree->descend($subst->{node}), 1);
         push @elems, $l => [$self->keyRewrite($l), $d] if defined $d;
     } 
 
@@ -1532,7 +1552,8 @@ sub findHooks($$$)
     {   my $match;
 
         $match++
-            if !$hook->{path} && !$hook->{id} && !$hook->{type};
+            if !$hook->{path} && !$hook->{id}
+            && !$hook->{type} && !$hook->{extends};
 
         if(!$match && $hook->{path})
         {   my $p = $hook->{path};
@@ -1559,6 +1580,10 @@ sub findHooks($$$)
                          } ref $t eq 'ARRAY' ? @$t : $t;
         }
 
+        if(!$match && defined $type && $hook->{extends})
+        {   $match++ if $self->{nss}->doesExtend($type, $hook->{extends});
+        }
+
         $match or next;
 
         foreach my $where ( qw/before replace after/ )
@@ -1606,5 +1631,11 @@ sub blocked($$$)
     $self->makeBlocked($path, $class, $type);
 }
 
+sub addTypeAttribute($$)
+{   my ($self, $type, $call) = @_;
+    $call;
+}
+
+#------------
 
 1;
