@@ -1,10 +1,10 @@
-# Copyrights 2006-2015 by [Mark Overmeer].
+# Copyrights 2006-2016 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
-# Pod stripped from pm file by OODoc 2.01.
+# Pod stripped from pm file by OODoc 2.02.
 package XML::Compile::Translate::Reader;
 use vars '$VERSION';
-$VERSION = '1.50';
+$VERSION = '1.52';
 
 use base 'XML::Compile::Translate';
 
@@ -311,7 +311,6 @@ sub makeAll($@)
 sub makeBlockHandler
 {   my ($self, $path, $label, $min, $max, $process, $kind, $multi) = @_;
 
-#warn "BLOCK $label $min $max";
     # flatten the HASH: when a block appears only once, there will
     # not be an additional nesting in the output tree.
     if($max ne 'unbounded' && $max==1)
@@ -1093,7 +1092,7 @@ sub makeHook($$$$$$$)
     return $r unless $before || $replace || $after;
 
     return sub { ($_[0]->node->localName => 'SKIPPED') }
-        if $replace && grep {$_ eq 'SKIP'} @$replace;
+        if $replace && grep $_ eq 'SKIP', @$replace;
 
     my @replace = $replace ? map $self->_decodeReplace($path,$_),@$replace : ();
     my @before  = $before  ? map $self->_decodeBefore($path,$_), @$before  : ();
@@ -1136,7 +1135,8 @@ sub _decodeReplace($$)
 {   my ($self, $path, $call) = @_;
     return $call if ref $call eq 'CODE';
 
-    error __x"labeled replace hook `{call}' undefined for READER", call=>$call;
+      $call eq 'XML_NODE'  ? sub { ($_[3] => $_[0]) }    # don't parse XML
+    : error __x"labeled replace hook `{call}' undefined for READER",call=>$call;
 }
 
 my %after = 
