@@ -1,11 +1,11 @@
-# Copyrights 2006-2014 by [Mark Overmeer].
+# Copyrights 2006-2016 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
-# Pod stripped from pm file by OODoc 2.01.
+# Pod stripped from pm file by OODoc 2.02.
 
 package XML::Compile::Translate::Template;
 use vars '$VERSION';
-$VERSION = '1.47';
+$VERSION = '1.52';
 
 use base 'XML::Compile::Translate';
 
@@ -474,10 +474,11 @@ sub makeAttributeFixed
 }
 
 sub makeSubstgroup
-{   my ($self, $path, $type, @do) = @_;
+{   my ($self, $path, $type, @todo) = @_;
 
     sub {
         my (@example_tags, $example_nest, %tags);
+        my @do    = @todo;
         my $group = $do[1][0];
 
         while(@do)
@@ -570,8 +571,8 @@ sub makeAnyElement
     bless sub { +$data }, 'ANY';
 }
 
-sub makeHook($$$$$$)
-{   my ($self, $path, $r, $tag, $before, $replace, $after) = @_;
+sub makeHook($$$$$$$)
+{   my ($self, $path, $r, $tag, $before, $replace, $after, $fulltype) = @_;
 
     return $r unless $before || $replace || $after;
 
