@@ -1,11 +1,11 @@
-# Copyrights 2006-2014 by [Mark Overmeer].
+# Copyrights 2006-2016 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
-# Pod stripped from pm file by OODoc 2.01.
+# Pod stripped from pm file by OODoc 2.02.
  
 package XML::Compile::Translate::Writer;
 use vars '$VERSION';
-$VERSION = '1.47';
+$VERSION = '1.52';
 
 use base 'XML::Compile::Translate';
 
@@ -17,7 +17,10 @@ use Log::Report   qw/xml-compile/;
 use List::Util    qw/first/;
 use Scalar::Util  qw/blessed weaken/;
 use XML::Compile::Util qw/pack_type unpack_type type_of_node SCHEMA2001i
-  odd_elements even_elements/;
+  SCHEMA2001 odd_elements even_elements/;
+
+use XML::LibXML;
+use Encode        qw/encode/;
 
 
 # Each action implementation returns a code reference, which will be
@@ -482,7 +485,6 @@ sub nil($)
 
 sub makeComplexElement
 {   my ($self, $path, $tag, $elems, $attrs, $any_attr,undef, $is_nillable) = @_;
-my @e = @$elems;
     my @elems   = odd_elements @$elems;
     my @attrs   = @$attrs;
     my $tags    = join ', ', grep defined
@@ -613,6 +615,7 @@ sub makeMixedElement
     my @attrs   = @$attrs;
     my @anya    = @$attrs_any;
     my $nilattr = $is_nillable ? $self->nil($path) : undef;
+    (my $locname = $tag) =~ s/.*\://;
 
     my $mixed = $self->{mixed_elements};
     if($mixed eq 'ATTRIBUTES') { ; }
@@ -644,14 +647,38 @@ sub makeMixedElement
           my $content = delete $copy->{_};
           defined $content or return;
 
-          if(UNIVERSAL::isa($content, 'XML::LibXML::Node')) {}
-          elsif($content eq 'NIL')
-          {   $content = $doc->createAttribute($nilattr => 'true')
+          #XXX there are no regression test for these options
+          my $node;
+          if(blessed $content && $content->isa('XML::LibXML::Node'))
+          {   $node    = $doc->importNode($content);
+          }
+          elsif($is_nillable && $content eq 'NIL')
+          {   # nillable element
+              $node    = $doc->createElement($tag);
+              $node->setAttribute($nilattr => 'true');
+          }
+          elsif($content =~ /\<.*?\>|\&\w+\;/)
+          {   # XXX I do not know a way to fill text nodes without getting
+              # entity encoding on them.  Apparently, libxml2 has a
+              # xmlStringTextNoenc, which is not provided in XML::LibXML.
+              # Now, we need the expensive roundtrip, via the parser.
+
+              my $frag = XML::LibXML->new
+                ->parse_balanced_chunk(encode utf8 => $content);
+
+              $node = $frag->firstChild;
+              if($node->localName ne $locname)
+              {   # <nodes>  -->   <$locname>$nodes</$locname>
+                  my $c = $node;
+                  $node = $doc->createElement($tag);
+                  $node->appendChild($frag);
+              }
           }
           else
-          {   $content = $doc->createTextNode($content);
+          {   # Plain text
+              $node    = $doc->createElement($tag);
+              $node->appendText($content);
           }
-          my $node = $doc->importNode($content);
 
           my @childs;
           for(my $i=0; $i<@attrs; $i+=2)
@@ -728,7 +755,7 @@ sub makeBuiltin
     $check
     ? ( defined $format
       ? sub { defined $_[1] or return undef;
-              my $value = $format->($_[1], $trans);
+              my $value = $format->($_[1], $trans, $path);
               return $value if defined $value && $check->($value);
               error __x$err, value => $value, type => $type, path => $path;
             }
@@ -737,7 +764,7 @@ sub makeBuiltin
             }
       )
     : ( defined $format
-      ? sub { defined $_[1] ? $format->($_[1], $trans) : undef }
+      ? sub { defined $_[1] ? $format->($_[1], $trans, $path) : undef }
       : sub { $_[1] }
       );
 }
@@ -984,7 +1011,7 @@ sub makeAnyElement
               elsif(index($label, ':') >= 0)
               {   (my $prefix, $local) = split ':', $label, 2;
                   my $match = first {$_->{prefix} eq $prefix} values %$prefs;
-                  my $ns    = $match ? $match->{uri} : undef;
+                  $ns    = $match ? $match->{uri} : undef;
                   $type  = pack_type $ns, $local;
               }
               else {next}  # not fully qualified, not an 'any'
@@ -1057,34 +1084,40 @@ sub makeXsiTypeSwitch($$$$)
     };
 }
 
-sub makeHook($$$$$$)
-{   my ($self, $path, $r, $tag, $before, $replace, $after) = @_;
+sub makeHook($$$$$$$)
+{   my ($self, $path, $r, $tag, $before, $replace, $after, $fulltype) = @_;
     return $r unless $before || $replace || $after;
 
-    error __x"writer only supports one production (replace) hook"
-        if $replace && @$replace > 1;
+    my $do_replace;
+    if($replace)
+    {   return sub {()} if grep $_ eq 'SKIP', @$replace;
 
-    return sub {()} if $replace && grep $_ eq 'SKIP', @$replace;
+        # Input for replace is Perl, output is XML... so we cannot stack them
+        error __x"writer only supports one replace hook (for {type})"
+          , type => $fulltype
+            if @$replace > 1;
 
-    my @replace = $replace ? map $self->_decodeReplace($path,$_), @$replace :();
-    my @before  = $before  ? map $self->_decodeBefore($path,$_),  @$before  :();
-    my @after   = $after   ? map $self->_decodeAfter($path,$_),   @$after   :();
+        $do_replace = $self->_decodeReplace($path, $replace->[0]);
+    }
+
+    my @do_before = $before ? map $self->_decodeBefore($path,$_), @$before :();
+    my @do_after  = $after  ? map $self->_decodeAfter($path,$_),  @$after  :();
 
     sub
     {  my ($doc, $val) = @_;
        defined $val or return;
-       foreach (@before)
-       {   $val = $_->($doc, $val, $path);
+       foreach (@do_before)
+       {   $val = $_->($doc, $val, $path, $fulltype);
            defined $val or return ();
        }
 
-       my $xml = @replace
-               ? $replace[0]->($doc, $val, $path, $tag, $r)
+       my $xml = $do_replace
+               ? $do_replace->($doc, $val, $path, $tag, $r, $fulltype)
                : $r->($doc, $val);
        defined $xml or return ();
 
-       foreach (@after)
-       {   $xml = $_->($doc, $xml, $path, $val);
+       foreach (@do_after)
+       {   $xml = $_->($doc, $xml, $path, $val, $fulltype);
            defined $xml or return ();
        }
 
@@ -1141,6 +1174,21 @@ sub makeBlocked($$$)
     : panic "blocking of $class for $type not implemented";
 }
 
+sub addTypeAttribute($$)
+{   my ($self, $type, $do) = @_;
+    my $xsi   = $self->_registerNSprefix(xsi => SCHEMA2001i, 1) . ':type';
+    my $xsd   = $self->_registerNSprefix(xsd => SCHEMA2001, 1);
+    my $typed = $self->prefixed($type);
+
+    sub {
+        my $r = $do->(@_);
+        $type && $r && UNIVERSAL::isa($r, 'XML::LibXML::Element') or return $r;
+        return $r if $r->getAttributeNS(SCHEMA2001i, 'type');
+        $r->setAttribute($xsi, $typed);
+        $r;
+    };
+}
+
+#------------
 
 1;
-
